"""OpenHubAPI URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.2/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.conf.urls import url
from django.contrib import admin
from django.urls import include
from rest_framework import routers
from django.urls import path
from data.views.views import HardwareViewSet, AccessoryViewSet, CalibrationViewSet, ChannelViewSet, HubViewSet, \
    IOViewSet
#
# # Routers provide an easy way of automatically determining the URL conf.
# router = routers.DefaultRouter()
# router.register(r'accessories', AccessoryViewSet)
# router.register(r'hardwares', HardwareViewSet)
# router.register(r'channel', ChannelViewSet)
# router.register(r'calibration', CalibrationViewSet)
# router.register(r'hubs', HubViewSet)
# router.register(r'io', IOViewSet)


# Wire up our API using automatic URL routing.
# Additionally, we include login URLs for the browsable API.
urlpatterns = [
    path('', include('data.urls')),
    ]



# Generated by Django 3.2.3 on 2021-06-09 21:09

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Accessory',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('category', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display_name', models.CharField(max_length=255, null=True)),
                ('aid', models.IntegerField()),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name='Calibration',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('type', models.CharField(max_length=255, null=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name='CalibrationConstants',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('type', models.CharField(max_length=255, null=True)),
                ('value', models.CharField(max_length=255, null=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name='Channel',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('channel_index', models.IntegerField(null=True)),
                ('type', models.CharField(max_length=15)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name='Hardware',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('type', models.CharField(choices=[('DHT22', 'Dht22'), ('MCP3008', 'Mcp3008'), ('ModProbe', 'Modprobe'), ('PiPico', 'Pipico'), ('VEML7700', 'Veml7700')], default='PiPico', max_length=10)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name='HardwareChannelTypes',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('hardware_type', models.CharField(max_length=10)),
                ('channel_type', models.CharField(max_length=10)),
            ],
        ),
        migrations.CreateModel(
            name='DHT22',
            fields=[
                ('hardware_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='data.hardware')),
                ('pin', models.IntegerField()),
            ],
            bases=('data.hardware',),
        ),
        migrations.CreateModel(
            name='MCP3008',
            fields=[
                ('hardware_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='data.hardware')),
                ('sck', models.IntegerField()),
                ('miso', models.IntegerField()),
                ('cs_pin', models.IntegerField()),
                ('num_channels', models.IntegerField()),
            ],
            bases=('data.hardware',),
        ),
        migrations.CreateModel(
            name='ModProbe',
            fields=[
                ('hardware_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='data.hardware')),
                ('base_dir', models.CharField(max_length=100, null=True)),
                ('file_name', models.CharField(max_length=100, null=True)),
            ],
            bases=('data.hardware',),
        ),
        migrations.CreateModel(
            name='PiPico',
            fields=[
                ('hardware_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='data.hardware')),
                ('pi_gpio_interrupt', models.IntegerField()),
            ],
            bases=('data.hardware',),
        ),
        migrations.CreateModel(
            name='VEML7700',
            fields=[
                ('hardware_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='data.hardware')),
                ('scl', models.IntegerField()),
                ('sda', models.IntegerField()),
            ],
            bases=('data.hardware',),
        ),
        migrations.CreateModel(
            name='HardwareConfig',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('type', models.CharField(max_length=255, null=True)),
                ('value', models.CharField(max_length=255, null=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('hardware', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='data.hardware')),
            ],
        ),
        migrations.AddIndex(
            model_name='hardware',
            index=models.Index(fields=['id'], name='data_hardwa_id_792ac4_idx'),
        ),
        migrations.AddField(
            model_name='channel',
            name='hardware',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='data.hardware'),
        ),
        migrations.AddField(
            model_name='calibrationconstants',
            name='calibration',
            field=models.ForeignKey(blank=True, on_delete=django.db.models.deletion.CASCADE, to='data.calibration'),
        ),
        migrations.AddField(
            model_name='calibration',
            name='accessory',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='data.accessory'),
        ),
        migrations.AddField(
            model_name='accessory',
            name='channels',
            field=models.ManyToManyField(to='data.Channel'),
        ),
        migrations.AddIndex(
            model_name='hardwareconfig',
            index=models.Index(fields=['id', 'hardware'], name='data_hardwa_id_9f4bd6_idx'),
        ),
        migrations.AddIndex(
            model_name='channel',
            index=models.Index(fields=['id', 'hardware'], name='data_channe_id_469160_idx'),
        ),
        migrations.AddIndex(
            model_name='calibrationconstants',
            index=models.Index(fields=['id'], name='data_calibr_id_0c5a5a_idx'),
        ),
        migrations.AddIndex(
            model_name='calibration',
            index=models.Index(fields=['id'], name='data_calibr_id_c3df15_idx'),
        ),
        migrations.AddIndex(
            model_name='accessory',
            index=models.Index(fields=['id'], name='data_access_id_df10ff_idx'),
        ),
    ]



# Generated by Django 3.2.5 on 2021-10-04 19:34

from django.db import migrations
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0030_remove_datatransformer_parent'),
    ]

    operations = [
        migrations.AddField(
            model_name='datatransformer',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='data.datatransformer'),
        ),
    ]



# chat/consumers.py
import json
from channels.generic.websocket import AsyncWebsocketConsumer, AsyncConsumer, WebsocketConsumer
import asyncio


class StartConsumer(AsyncWebsocketConsumer):
    def __init__(self, *args, **kwargs):
        super(StartConsumer, self).__init__(*args, **kwargs)

    # groups = ["broadcast"]

    async def connect(self):
        await self.accept()
        self.reader, self.writer = await asyncio.open_connection(
            '127.0.0.1', 8888)
        # Called on connection.
        # To accept the connection call:

    async def send_to_core(self, text_data):
        print(f'Send: {text_data!r}')
        self.writer.write((text_data+'\n').encode())
        await self.writer.drain()

        data = await self.reader.readline()
        self.send(text_data=data.decode())

        print(f'Received: {data.decode()!r}')

    async def receive(self, text_data=None, bytes_data=None):
        print('receive')
        await self.send_to_core(text_data)

    async def disconnect(self, close_code):
        print('Close the connection')
        self.writer.close()
        await self.writer.wait_closed()
        self.close()



# Generated by Django 3.2.5 on 2021-10-03 00:50

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0023_auto_20211002_2232'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='datatransformer',
            name='data_transformer_child',
        ),
        migrations.AlterField(
            model_name='datatransformer',
            name='data_transformer_parent',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='data.datatransformer'),
        ),
    ]



"""
Django settings for OpenHubAPI project.

Generated by 'django-admin startproject' using Django 3.2.3.

For more information on this file, see
https://docs.djangoproject.com/en/3.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.2/ref/settings/
"""

from pathlib import Path

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'django-insecure-9o#jxp0=)b%9*3l2s=eslnjhau-+-!e&%-cwc&5s1-rjd2eehb'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ['*']
# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'rest_framework',
    'polymorphic',
    # 'channels',
    'data',
    'mptt'
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'OpenHubAPI.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [BASE_DIR / '/templates']
        ,
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'data.context_processors.basetest'
            ],
        },
    },
]

CHANNEL_LAYERS = {
    "default": {
        "BACKEND": "channels.layers.InMemoryChannelLayer"
    }
}

WSGI_APPLICATION = 'OpenHubAPI.wsgi.application'

# Database
# https://docs.djangoproject.com/en/3.2/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': '/home/openhubdaemon/db.sqlite3',
    }
}

# Password validation
# https://docs.djangoproject.com/en/3.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

# Internationalization
# https://docs.djangoproject.com/en/3.2/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.2/howto/static-files/

STATIC_URL = '/static/'
STATICFILES_DIRS = (str(BASE_DIR.joinpath('static')),)  # new
STATIC_ROOT = str(BASE_DIR.joinpath('staticfiles'))  # new

# Default primary key field type
# https://docs.djangoproject.com/en/3.2/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

REST_FRAMEWORK = {'DEFAULT_PERMISSION_CLASSES': [
    'rest_framework.permissions.AllowAny'
]}

ASGI_APPLICATION = "OpenHubAPI.asgi.application"

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
        },
    },
    'root': {
        'handlers': ['console'],
        'level': 'WARNING',
    },
}

LOGIN_REDIRECT_URL='/hubs'



# Generated by Django 3.2.4 on 2021-07-12 20:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0010_auto_20210712_2038'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='pigpio',
            name='channel_index',
        ),
        migrations.RemoveField(
            model_name='pipicoanalogio',
            name='channel_index',
        ),
    ]



# Generated by Django 3.2.5 on 2021-10-05 21:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0032_alter_datatransformer_accessory'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='hardwaredatatransformer',
            name='data_hardwa_id_c5fb61_idx',
        ),
        migrations.RemoveIndex(
            model_name='hardwarestats',
            name='data_hardwa_id_5b03a1_idx',
        ),
        migrations.AlterField(
            model_name='datatransformer',
            name='type',
            field=models.CharField(blank=True, default=None, max_length=15, null=True),
        ),
        migrations.RemoveField(
            model_name='hardwaredatatransformer',
            name='hardware',
        ),
        migrations.AddField(
            model_name='hardwaredatatransformer',
            name='hardware',
            field=models.ManyToManyField(to='data.Channel'),
        ),
        migrations.RemoveField(
            model_name='hardwarestats',
            name='hardware',
        ),
        migrations.AddField(
            model_name='hardwarestats',
            name='hardware',
            field=models.ManyToManyField(to='data.Channel'),
        ),
        migrations.AddIndex(
            model_name='hardwaredatatransformer',
            index=models.Index(fields=['id', 'data_transformer'], name='data_hardwa_id_d8bcf2_idx'),
        ),
        migrations.AddIndex(
            model_name='hardwarestats',
            index=models.Index(fields=['id'], name='data_hardwa_id_4060f5_idx'),
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-21 04:57

from django.db import migrations

def gen(apps, schema_editor):
    from data.models.models import AccessoryType, HardwareChannelTypes, Category

    categories = [('Other', 1), ('Bridge', 2), ('Fan', 3), ('Garage Door Opener', 4), ('Lightbulb', 5),
                  ('Door Lock', 6), ('Outlet', 7), ('Switch', 8), ('Thermostat', 9), ('Sensor', 10),
                  ('Alarm system', 11), ('Door', 12), ('Window', 13), ('Window Covering', 14),
                  ('Programmable Switch', 15), ('Range Extender', 16), ('Camera', 17), ('Video Door Bell', 18),
                  ('Air Purifier', 19), ('Heater', 20), ('Air Conditioner', 21), ('Humidifier', 22),
                  ('Dehumidifier', 23), ('Speaker', 26), ('Sprinkler', 28), ('Faucet', 29), ('Shower head', 30),
                  ('Television', 31), ('Target Controller', 32)]

    for (name, enum) in categories:
        print(name)
        print(str(enum))
        cat = Category.objects.create()
        cat.name = name
        cat.enum = enum
        cat.save()

    types = ['AirTemperatureSensor',
             'ETapeSensor',
             'HumiditySensor',
             'LiquidLevelSensor',
             'PressureSensor',
             'Pump',
             'SoilMoistureSensor',
             'SoilTemperatureSensor']

    for type in types:
        cat = AccessoryType.objects.create()
        cat.type = type
        cat.save()

    hardware_channel_types = [('PiPico', 'PiPicoAnalog'), ('PiPico', 'PiPicoPump')]

    for (hard, channel) in hardware_channel_types:
        cat = HardwareChannelTypes.objects.create()
        cat.hardware_type = hard
        cat.channel_type = channel
        cat.save()


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0013_auto_20210713_1524'),
    ]

    operations = [
        migrations.RunPython(gen, reverse_code=migrations.RunPython.noop),
    ]



# Generated by Django 3.2.4 on 2021-07-12 16:49

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('data', '0007_pipico_serial_com'),
    ]

    operations = [
        migrations.CreateModel(
            name='HardwareIO',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('child_channel', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='data.channel')),
                ('child_hardware', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='data.hardware')),
                ('hub', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='data.hub')),
                ('parent_hardware', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='data_hardwareio_related', related_query_name='data_hardwareios', to='data.hardware')),
                ('polymorphic_ctype', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='polymorphic_data.hardwareio_set+', to='contenttypes.contenttype')),
            ],
        ),
        migrations.AlterField(
            model_name='hardwarechanneltypes',
            name='channel_type',
            field=models.CharField(max_length=20),
        ),
        migrations.AlterField(
            model_name='hardwarechanneltypes',
            name='hardware_type',
            field=models.CharField(max_length=20),
        ),
        migrations.CreateModel(
            name='DeviceFileIo',
            fields=[
                ('hardwareio_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='data.hardwareio')),
                ('base_dir', models.CharField(max_length=100, null=True)),
                ('device_file', models.CharField(max_length=100, null=True)),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
            bases=('data.hardwareio',),
        ),
        migrations.CreateModel(
            name='I2cIo',
            fields=[
                ('hardwareio_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='data.hardwareio')),
                ('scl', models.IntegerField()),
                ('sda', models.IntegerField()),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
            bases=('data.hardwareio',),
        ),
        migrations.CreateModel(
            name='MCPAnalogIo',
            fields=[
                ('hardwareio_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='data.hardwareio')),
                ('channel_index', models.IntegerField()),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
            bases=('data.hardwareio',),
        ),
        migrations.CreateModel(
            name='PiGpio',
            fields=[
                ('hardwareio_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='data.hardwareio')),
                ('pin', models.IntegerField()),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
            bases=('data.hardwareio',),
        ),
        migrations.CreateModel(
            name='PiPicoAnalogIo',
            fields=[
                ('hardwareio_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='data.hardwareio')),
                ('channel_index', models.IntegerField()),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
            bases=('data.hardwareio',),
        ),
        migrations.CreateModel(
            name='PwmIo',
            fields=[
                ('hardwareio_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='data.hardwareio')),
                ('pwm_pin', models.IntegerField()),
                ('duty_max', models.IntegerField()),
                ('duty_min', models.IntegerField()),
                ('freq', models.IntegerField()),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
            bases=('data.hardwareio',),
        ),
        migrations.CreateModel(
            name='SerialIo',
            fields=[
                ('hardwareio_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='data.hardwareio')),
                ('port', models.CharField(max_length=100)),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
            bases=('data.hardwareio',),
        ),
        migrations.CreateModel(
            name='SPIIo',
            fields=[
                ('hardwareio_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='data.hardwareio')),
                ('sck', models.IntegerField()),
                ('miso', models.IntegerField()),
                ('mosi', models.IntegerField()),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
            bases=('data.hardwareio',),
        ),
        migrations.AddIndex(
            model_name='hardwareio',
            index=models.Index(fields=['id', 'parent_hardware', 'child_channel'], name='data_hardwa_id_5685de_idx'),
        ),
    ]



import mptt
from django.db import models
import uuid
from django.utils import timezone
import time
from polymorphic.models import PolymorphicModel
from mptt.models import MPTTModel, TreeForeignKey


class Hub(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
    ip = models.URLField()
    category = models.CharField(max_length=255, null=True)
    type = models.CharField(max_length=255, null=True)
    display_name = models.CharField(max_length=255, null=True)
    aid = models.IntegerField()

    updated_at = models.DateTimeField(auto_now=True)
    created_at = models.DateTimeField(auto_now_add=True)

    def save(self, *args, **kwargs):
        if not self.created_at:
            self.created_at = timezone.now()
        self.updated_at = timezone.now()
        return super().save(*args, **kwargs)

    class Meta:
        indexes = [
            models.Index(fields=['id']),
        ]


class Hardware(PolymorphicModel):
    class Type(models.TextChoices):
        DHT22 = 'DHT22'
        MCP3008 = 'MCP3008'
        ModProbe = 'ModProbe'
        PiPico = 'PiPico'
        VEML7700 = 'VEML7700'

    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
    type = models.CharField(
        max_length=10,
        choices=Type.choices,
        default=Type.PiPico,
    )

    updated_at = models.DateTimeField(auto_now=True)
    created_at = models.DateTimeField(auto_now_add=True)
    hub = models.ForeignKey(Hub, on_delete=models.CASCADE, null=False)

    def save(self, *args, **kwargs):
        if not self.created_at:
            self.created_at = timezone.now()
        self.updated_at = timezone.now()

        return super().save(*args, **kwargs)

    class Meta:
        indexes = [
            models.Index(fields=['id']),
        ]


class PiPico(Hardware):
    serial_com = models.CharField(max_length=100, null=True)
    pi_gpio_interrupt = models.IntegerField()


class DHT22(Hardware):
    pin = models.IntegerField()


class MCP3008(Hardware):
    sck = models.IntegerField()
    miso = models.IntegerField()
    cs_pin = models.IntegerField()
    num_channels = models.IntegerField()


class ModProbe(Hardware):
    base_dir = models.CharField(max_length=100, null=True)
    file_name = models.CharField(max_length=100, null=True)


class VEML7700(Hardware):
    scl = models.IntegerField()
    sda = models.IntegerField()


class PMSA0031(Hardware):
    scl = models.IntegerField()
    sda = models.IntegerField()
    reset = models.IntegerField()


class HardwareChannelTypes(models.Model):
    hardware_type = models.CharField(
        max_length=20
    )
    channel_type = models.CharField(
        max_length=20
    )


class Channel(models.Model):
    def __str__(self):
        return self.get_name_display()

    def get_name_display(self):
        name = ''
        if self.hardware is not None:
            name = name + self.hardware.type + ' / '
        name = name + self.type + ' / ' + str(self.channel_index) + ' / ' + str(self.id)
        return name

    class Type(models.TextChoices):
        DHT22Humidity = 'DHT22Humidity'
        DHT22Temp = 'DHT22Temp'
        MCP3008Analog = 'MCP3008Analog'
        ModProbeTemp = 'ModProbeTemp'
        PiPicoACAnalog = 'PiPicoACAnalog'
        PiPicoAnalog = 'PiPicoAnalog'
        PiPicoPump = 'PiPicoPump'
        PiPicoRelay = 'PiPicoRelay'
        VEML7700Light = 'VEML7700Light'
        VEML7700Lux = 'VEML7700Lux'

    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
    channel_index = models.IntegerField(null=True)
    type = models.CharField(
        max_length=15
    )
    hardware = models.ForeignKey(Hardware, on_delete=models.CASCADE, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True)
    created_at = models.DateTimeField(auto_now_add=True)
    hub = models.ForeignKey(Hub, on_delete=models.CASCADE, null=False)
    keep_statistics = models.BooleanField(null=True, blank=True, default=False)

    def save(self, *args, **kwargs):
        if not self.created_at:
            self.created_at = timezone.now()
        self.updated_at = timezone.now()

        return super().save(*args, **kwargs)

    class Meta:
        indexes = [
            models.Index(fields=['id', 'hardware']),
        ]


class HardwareIO(PolymorphicModel):
    SPI = 'SPI'
    Serial = 'Serial'
    PWM = 'PWM'
    I2C = 'I2C'
    DeviceFile = 'Device File'
    MCPChannel = 'MCP Channel'
    PiPicoACAnalog = 'Pi Pico AC Analog'
    PiPicoAnalog = 'Pi Pico Analog'
    PiGPIO = 'Pi GPIO'
    hardware_io_types_choices = [(SPI, SPI), (Serial, Serial), (PWM, PWM), (I2C, I2C), (DeviceFile, DeviceFile),
                                 (MCPChannel, MCPChannel), (PiPicoAnalog, PiPicoAnalog),
                                 (PiPicoACAnalog, PiPicoACAnalog), (PiGPIO, PiGPIO)]

    label = models.CharField(max_length=255, null=True)

    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
    parent_hardware = models.ForeignKey(Hardware, on_delete=models.CASCADE, blank=True, null=True,
                                        related_name="%(app_label)s_%(class)s_related",
                                        related_query_name="%(app_label)s_%(class)ss")
    child_hardware = models.ForeignKey(Hardware, on_delete=models.CASCADE, blank=True, null=True)
    child_channel = models.ForeignKey(Channel, on_delete=models.CASCADE, blank=True, null=True)
    type = models.CharField(max_length=25, choices=hardware_io_types_choices, default=PiPicoAnalog)

    updated_at = models.DateTimeField(auto_now=True)
    created_at = models.DateTimeField(auto_now_add=True)
    hub = models.ForeignKey(Hub, on_delete=models.CASCADE, null=False)

    def save(self, *args, **kwargs):
        if not self.created_at:
            self.created_at = timezone.now()
        self.updated_at = timezone.now()

        return super().save(*args, **kwargs)

    class Meta:
        indexes = [
            models.Index(fields=['id', 'parent_hardware', 'child_channel']),
        ]


class PiPins(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
    hardware_io = models.ForeignKey(HardwareIO, on_delete=models.CASCADE, blank=True, null=True)
    pin = models.IntegerField()

    class Meta:
        indexes = [
            models.Index(fields=['hardware_io']),
        ]


class SPIIo(HardwareIO):
    sck = models.IntegerField()
    miso = models.IntegerField()
    mosi = models.IntegerField()


class SerialIo(HardwareIO):
    port = models.CharField(max_length=100)


class PwmIo(HardwareIO):
    en = models.IntegerField()
    duty_max = models.IntegerField()
    duty_min = models.IntegerField()
    freq = models.IntegerField()


class I2cIo(HardwareIO):
    scl = models.IntegerField()
    sda = models.IntegerField()


class DeviceFileIo(HardwareIO):
    base_dir = models.CharField(max_length=100, null=True)
    device_file = models.CharField(max_length=100, null=True)


class MCPAnalogIo(HardwareIO):
    channel_index = models.IntegerField()


class PiPicoAnalogIo(HardwareIO):
    pin = models.IntegerField()


class PiPicoACAnalogIo(HardwareIO):
    pin = models.IntegerField()


class PiGpio(HardwareIO):
    pin = models.IntegerField()


class Accessory(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
    hub = models.ForeignKey(Hub, on_delete=models.CASCADE, null=False)

    category = models.CharField(max_length=255, null=True)
    type = models.CharField(max_length=255, null=True)
    display_name = models.CharField(max_length=255, null=True)
    aid = models.IntegerField()
    channels = models.ManyToManyField(Channel)

    updated_at = models.DateTimeField(auto_now=True)
    created_at = models.DateTimeField(auto_now_add=True)

    def save(self, *args, **kwargs):
        if not self.created_at:
            self.created_at = timezone.now()
        self.updated_at = timezone.now()
        return super().save(*args, **kwargs)

    @property
    def has_data_transformer(self):
        return self.datatransformer_set.count() > 0

    @property
    def datatransformer(self):
        return self.datatransformer_set.first().get_root()

    class Meta:
        indexes = [
            models.Index(fields=['id']),
        ]


class HardwareConfig(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
    type = models.CharField(max_length=255, null=True)
    value = models.CharField(max_length=255, null=True)
    hardware = models.ForeignKey(Hardware, on_delete=models.CASCADE, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True)
    created_at = models.DateTimeField(auto_now_add=True)

    def save(self, *args, **kwargs):
        if not self.created_at:
            self.created_at = timezone.now()
        self.updated_at = timezone.now()

        return super().save(*args, **kwargs)

    class Meta:
        indexes = [
            models.Index(fields=['id', 'hardware']),
        ]


class HardwareStats(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
    hardware = models.ManyToManyField(Channel)

    type = models.CharField(max_length=255, null=True)
    value = models.FloatField(null=True)
    updated_at = models.DateTimeField(auto_now=True)
    created_at = models.DateTimeField(auto_now_add=True)

    def save(self, *args, **kwargs):
        if not self.created_at:
            self.created_at = timezone.now()
        self.updated_at = timezone.now()

        return super().save(*args, **kwargs)

    class Meta:
        indexes = [
            models.Index(fields=['id']),
        ]


class DataTransformerTypes(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
    type = models.CharField(
        max_length=15, blank=True, null=True, default=None
    )

class DataTransformer(MPTTModel):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
    accessory = models.ForeignKey(
        Accessory,
        on_delete=models.CASCADE,
        blank=True,
        null=True
    )


    type = models.ForeignKey(DataTransformerTypes, on_delete=models.DO_NOTHING, blank=True, null=True, default=None)

    parent = TreeForeignKey('self', on_delete=models.CASCADE, null=True, blank=True, related_name='children')
    updated_at = models.DateTimeField(blank=True,auto_now=True)
    created_at = models.DateTimeField(blank=True,auto_now_add=True)

    def save(self, *args, **kwargs):
        if not self.created_at:
            self.created_at = timezone.now()
        self.updated_at = timezone.now()

        return super().save(*args, **kwargs)

    class Meta:
        indexes = [
            models.Index(fields=['id', 'accessory']),
        ]

    class MPTTMeta:
        order_insertion_by = ['id']


class DataTransformerConstant(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4,blank=True,null=False)
    data_transformer = models.ForeignKey(DataTransformer, related_name='data_transformer_constants', on_delete=models.CASCADE, blank=True, null=True)
    index = models.IntegerField(blank=True, null=True, default=None)
    value = models.FloatField(blank=True,null=False)
    updated_at = models.DateTimeField(auto_now=True)
    created_at = models.DateTimeField(auto_now_add=True)

    def save(self, *args, **kwargs):
        if not self.created_at:
            self.created_at = timezone.now()
        self.updated_at = timezone.now()

        return super().save(*args, **kwargs)

    class Meta:
        indexes = [
            models.Index(fields=['id', 'data_transformer']),
        ]


class HardwareDataTransformer(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
    hardware = models.ManyToManyField(Channel,blank=True)
    data_transformer = models.ForeignKey(DataTransformer, on_delete=models.CASCADE, blank=True, null=True)
    index = models.IntegerField(blank=True, null=True, default=None)
    updated_at = models.DateTimeField(blank=True,auto_now=True)
    created_at = models.DateTimeField(blank=True,auto_now_add=True)

    def save(self, *args, **kwargs):
        if not self.created_at:
            self.created_at = timezone.now()
        self.updated_at = timezone.now()

        return super().save(*args, **kwargs)

    class Meta:
        indexes = [
            models.Index(fields=['id', 'data_transformer']),
        ]


class HardwareStatsDataTransformer(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4,blank=True,null=False)
    hardware_stat = models.ForeignKey(HardwareStats, on_delete=models.CASCADE, blank=True, null=True, default=None)
    data_transformer = models.ForeignKey(DataTransformer, on_delete=models.CASCADE, blank=True, null=True)
    index = models.IntegerField(blank=True, null=True, default=None)
    updated_at = models.DateTimeField(blank=True,auto_now=True)
    created_at = models.DateTimeField(blank=True,auto_now_add=True)

    def save(self, *args, **kwargs):
        if not self.created_at:
            self.created_at = timezone.now()
        self.updated_at = timezone.now()

        return super().save(*args, **kwargs)

    class Meta:
        indexes = [
            models.Index(fields=['id', 'data_transformer']),
        ]


class Calibration(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
    type = models.CharField(max_length=255, null=True)
    accessory = models.ForeignKey(Accessory, on_delete=models.CASCADE, blank=True, null=True, default=None)

    updated_at = models.DateTimeField(auto_now=True)
    created_at = models.DateTimeField(auto_now_add=True)

    def save(self, *args, **kwargs):
        if not self.created_at:
            self.created_at = timezone.now()
        self.updated_at = timezone.now()

        return super().save(*args, **kwargs)

    class Meta:
        indexes = [
            models.Index(fields=['id'])
        ]


class CalibrationConstants(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
    type = models.CharField(max_length=255, null=True)
    value = models.CharField(max_length=255, null=True)
    updated_at = models.DateTimeField(auto_now=True)
    created_at = models.DateTimeField(auto_now_add=True)
    calibration = models.ForeignKey(Calibration, on_delete=models.CASCADE, blank=True, null=False)

    def save(self, *args, **kwargs):
        if not self.created_at:
            self.created_at = timezone.now()
        self.updated_at = timezone.now()

        return super().save(*args, **kwargs)

    class Meta:
        indexes = [
            models.Index(fields=['id']),
        ]


class Category(models.Model):
    name = models.CharField(max_length=255, null=True)
    enum = models.IntegerField(null=True)


class AccessoryType(models.Model):
    type = models.CharField(max_length=255, null=True)



# Generated by Django 3.2.4 on 2021-06-10 23:29

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0002_auto_20210610_1805'),
    ]

    operations = [
        migrations.CreateModel(
            name='Hub',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('ip', models.URLField()),
                ('category', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display_name', models.CharField(max_length=255, null=True)),
                ('aid', models.IntegerField()),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.AddIndex(
            model_name='hub',
            index=models.Index(fields=['id'], name='data_hub_id_055d65_idx'),
        ),
        migrations.AddField(
            model_name='accessory',
            name='hub',
            field=models.ForeignKey(default='2d881600-8fb2-4663-a12e-5ffca01539cf', on_delete=django.db.models.deletion.CASCADE, to='data.hub'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='channel',
            name='hub',
            field=models.ForeignKey(default='2d881600-8fb2-4663-a12e-5ffca01539cf', on_delete=django.db.models.deletion.CASCADE, to='data.hub'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='hardware',
            name='hub',
            field=models.ForeignKey(default='2d881600-8fb2-4663-a12e-5ffca01539cf', on_delete=django.db.models.deletion.CASCADE, to='data.hub'),
            preserve_default=False,
        ),
    ]



import json
from collections import defaultdict

from django.core import serializers as django_serializers
from django.forms import formset_factory, inlineformset_factory
from django.http import JsonResponse, HttpResponse, QueryDict
from django.shortcuts import render, get_object_or_404
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import ensure_csrf_cookie, csrf_exempt
from polymorphic.models import PolymorphicTypeInvalid
from rest_framework import viewsets, renderers
from rest_framework.decorators import renderer_classes, api_view
from rest_framework.renderers import TemplateHTMLRenderer, JSONRenderer
from rest_framework.response import Response

from data.forms.forms import HardwareForm, HardwareDHT22Form, HardwareMCP3008Form, HardwareModProbeForm, \
    HardwarePiPicoForm, HardwareVEML7700Form, HardwareTypeForm, ChannelForm, HardwareConfigForm, AccessoryForm, \
    CalibrationForm, CalibrationConstantForm, HubForm, HardwareIOTypeForm, SPIIoForm, PwmIoForm, SerialIoForm, \
    I2cIoForm, DeviceFileIoForm, MCPAnalogIoForm, PiPicoACAnalogIoForm, PiPicoAnalogIoForm, PiGpioForm, HardwareIoForm, \
    HardwarePMSA0031Form
from data.models.models import Hardware, DHT22, MCP3008, ModProbe, PiPico, VEML7700, Accessory, Calibration, \
    CalibrationConstants, Channel, Hub, HardwareIO
from data.serializers.serializers import HardwareSerializer, ChannelSerializer, AccessorySerializer, \
    CalibrationSerializer, HubSerializer, HardwareIOSerializer

SPI = 'SPI'
Serial = 'Serial'
PWM = 'PWM'
I2C = 'I2C'
DeviceFile = 'Device File'
MCPChannel = 'MCP Channel'
PiPicoAnalog = 'Pi Pico Analog'
PiPicoACAnalog = 'Pi Pico AC Analog'
PiGPIO = 'Pi GPIO'


class HardwareViewSet(viewsets.ModelViewSet):
    queryset = Hardware.objects.all()
    serializer_class = HardwareSerializer
    renderer_classes = (renderers.JSONRenderer, renderers.TemplateHTMLRenderer)

    def updateHardware(request):
        if request.is_ajax and request.method == "POST":
            # get the form data
            hardware = Hardware.objects.get(pk=request.POST['initial-id'])

            form = HardwareForm(data=request.POST, instance=hardware)
            hardware_type = form.data['type']
            if hardware_type == 'DHT22':
                form = HardwareDHT22Form(data=request.POST, instance=hardware)
            if hardware_type == 'MCP3008':
                form = HardwareMCP3008Form(data=request.POST, instance=hardware)
            if hardware_type == 'ModProbe':
                form = HardwareModProbeForm(data=request.POST, instance=hardware)
            if hardware_type == 'PiPico':
                form = HardwarePiPicoForm(data=request.POST, instance=hardware)
            if hardware_type == 'VEML7700':
                form = HardwareVEML7700Form(data=request.POST, instance=hardware)
            if hardware_type == 'PMSA0031':
                form = HardwarePMSA0031Form(data=request.POST, instance=hardware)
            # save the data and after fetch the object in instance
            if form.is_valid():
                instance = form.save()
                print(str(instance))
                # serialize in new friend object in json
                ser_instance = django_serializers.serialize('json', [instance, ])
                print(str(ser_instance))
                # send to client side.
                return JsonResponse({"instance": ser_instance}, status=200)

    def getHardwareTypeForm(request):
        form = HardwareTypeForm(request.POST)
        hardware_type = form.data['type']
        if hardware_type == 'DHT22':
            form = HardwareDHT22Form(initial={'type': 'DHT22'})
        if hardware_type == 'MCP3008':
            form = HardwareMCP3008Form(initial={'type': 'MCP3008'})
        if hardware_type == 'ModProbe':
            form = HardwareModProbeForm(initial={'type': 'ModProbe'})
        if hardware_type == 'PiPico':
            form = HardwarePiPicoForm(initial={'type': 'PiPico'})
        if hardware_type == 'VEML7700':
            form = HardwareVEML7700Form(initial={'type': 'VEML7700'})
        if hardware_type == 'PMSA0031':
            form = HardwarePMSA0031Form(initial={'type': 'PMSA0031'})
        return HttpResponse(form.as_p())

    def getHardwareIoForm(request):
        form = HardwareIOTypeForm(request.POST)
        hardware_io_type = form.data['type']
        initial = {}
        if 'parent_hardware' in form.data.keys():
            initial['parent_hardware'] = form.data['parent_hardware']
        if 'child_hardware' in form.data.keys():
            initial['child_hardware'] = form.data['child_hardware']
        if 'child_channel' in form.data.keys():
            initial['child_channel'] = form.data['child_channel']
        if 'child_channel' in form.data.keys():
            initial['hub'] = form.data['hub']
        if 'type' in form.data.keys():
            initial['type'] = form.data['type']

        if hardware_io_type == SPI:
            form = SPIIoForm(initial=initial)
            form.type = SPI

        if hardware_io_type == PWM:
            form = PwmIoForm(initial=initial)
            form.type = PWM

        if hardware_io_type == Serial:
            form = SerialIoForm(initial=initial)
            form.type = Serial

        if hardware_io_type == I2C:
            form = I2cIoForm(initial=initial)
            form.type = I2C

        if hardware_io_type == DeviceFile:
            form = DeviceFileIoForm(initial=initial)
            form.type = DeviceFile

        if hardware_io_type == MCPChannel:
            form = MCPAnalogIoForm(initial=initial)
            form.type = MCPChannel

        if hardware_io_type == PiPicoAnalog:
            form = PiPicoAnalogIoForm(initial=initial)
            form.type = PiPicoAnalog

        if hardware_io_type == PiPicoACAnalog:
            form = PiPicoACAnalogIoForm(initial=initial)
            form.type = PiPicoACAnalog

        if hardware_io_type == PiGPIO:
            form = PiGpioForm(initial=initial)
            form.type = PiGPIO

        return HttpResponse(form.as_p())

    def list(self, request, *args, **kwargs):
        print("list")

        response = super(HardwareViewSet, self).list(request, *args, **kwargs)
        form_type = HardwareTypeForm(initial={'type': 'PiPico'})
        hardware_form = HardwarePiPicoForm(initial={'type': 'PiPico'})
        hardware = Hardware.objects.all()

        serialized_hardware = HardwareSerializer().to_json_array(hardware)
        if request.accepted_renderer.format == 'html':
            return Response({"form": hardware_form, "type_form": form_type, "hardwares": hardware},
                            template_name='hardwares.html')
            # return Response({"hardwares": hardware}, template_name='hardwares.html')
        return response

    def getChannels(self, hardware_id):
        channels = Hardware.objects.get(pk=str(hardware_id)).channel_set.all()
        serialized_hardware = ChannelSerializer(many=True).to_representation(channels)

        # if request.accepted_renderer.format == 'html':
        return JsonResponse(serialized_hardware, status=200, safe=False)

    def getConfig(self, hardware_id):
        hardware_configs = Hardware.objects.get(pk=str(hardware_id)).hardwareconfig_set.all()
        config = {}
        for hardware_config in hardware_configs:
            config[hardware_config.type] = hardware_config.value
        # if request.accepted_renderer.format == 'html':
        return JsonResponse({"config": config}, status=200)

    def indexView(request):
        print("indexView")
        form = HardwareForm()
        hardware = Hardware.objects.all()
        return render(request, "hardwares.html", {"form": form, "hardware": hardware})

    SPI = 'SPI'
    Serial = 'Serial'
    PWM = 'PWM'
    I2C = 'I2C'
    DeviceFile = 'Device File'
    MCPChannel = 'MCP Channel'
    PiPicoAnalog = 'Pi Pico Analog'
    PiPicoACAnalog = 'Pi Pico AC Analog'
    PiGPIO = 'Pi GPIO'

    def postHardwareIO(request):
        # request should be ajax and method should be POST.
        if request.is_ajax and request.method == "POST":
            # get the form data
            hardware_io_form = HardwareIoForm(request.POST)
            hardware_io_type = hardware_io_form.data['type']
            if hardware_io_type == SPI:
                hardware_form = SPIIoForm(request.POST)
            if hardware_io_type == Serial:
                hardware_form = SerialIoForm(request.POST)
            if hardware_io_type == PWM:
                hardware_form = PwmIoForm(request.POST)
            if hardware_io_type == I2C:
                hardware_form = I2cIoForm(request.POST)
            if hardware_io_type == DeviceFile:
                hardware_form = DeviceFileIoForm(request.POST)
            if hardware_io_type == MCPChannel:
                hardware_form = MCPAnalogIoForm(request.POST)
            if hardware_io_type == PiPicoAnalog:
                hardware_form = PiPicoAnalogIoForm(request.POST)
            if hardware_io_type == PiPicoACAnalog:
                hardware_form = PiPicoACAnalogIoForm(request.POST)
            if hardware_io_type == PiGPIO:
                hardware_form = PiGpioForm(request.POST)
            # save the data and after fetch the object in instance
            if hardware_form.is_valid():
                instance = hardware_form.save()
                # serialize in new friend object in json
                ser_instance = django_serializers.serialize('json', [instance, ])
                # send to client side.
                return JsonResponse({"instance": ser_instance}, status=200)
            else:
                # some form errors occured.
                return JsonResponse({"error": hardware_form.errors}, status=400)

        # some error occured
        return JsonResponse({"error": ""}, status=400)

    def postHardware(request):
        # request should be ajax and method should be POST.
        if request.is_ajax() and request.method == "POST":
            # get the form data
            hardware_form = HardwareForm(request.POST)
            hardware_type = request.POST['type']
            if hardware_type == 'DHT22':
                hardware_form = HardwareDHT22Form(request.POST)
            if hardware_type == 'MCP3008':
                hardware_form = HardwareMCP3008Form(request.POST)
            if hardware_type == 'ModProbe':
                hardware_form = HardwareModProbeForm(request.POST)
            if hardware_type == 'PiPico':
                hardware_form = HardwarePiPicoForm(request.POST)
            if hardware_type == 'VEML7700':
                hardware_form = HardwareVEML7700Form(request.POST)
            if hardware_type == 'PMSA0031':
                form = HardwarePMSA0031Form(request.POST)

            # save the data and after fetch the object in instance
            if hardware_form.is_valid():
                instance = hardware_form.save()
                # serialize in new friend object in json
                ser_instance = django_serializers.serialize('json', [instance, ])
                # send to client side.
                return JsonResponse({"instance": ser_instance}, status=200)
            else:
                # some form errors occured.
                return JsonResponse({"error": hardware_form.errors}, status=400)
        elif request.method == "POST":
            import json
            body_json = json.loads(request.body.decode('utf-8'))
            pi_pico = PiPico()
            pi_pico.id = body_json['id']
            pi_pico.type = body_json['type']
            pi_pico.hub = Hub.objects.get(pk=body_json['hub'])
            pi_pico.pi_gpio_interrupt = 1
            pi_pico.save()
            ser_instance = django_serializers.serialize('json', [pi_pico, ])

            return JsonResponse({"instance": ser_instance}, status=200)
        # some error occured
        return JsonResponse({"error": ""}, status=400)

    def retrieve(self, request, *args, **kwargs):
        print('retrieve')
        print(str(request.path_info))
        hardware_form = HardwareForm(instance=Hardware.objects.get(**kwargs))
        hardware_type = Hardware.objects.get(**kwargs).type

        # hardware_io = HardwareIO.objects.get({'child_hardware':Hardware.objects.get(**kwargs).id})
        hardware_io_type_form = HardwareIOTypeForm(
            initial={'child_hardware': Hardware.objects.get(**kwargs), 'hub': Hardware.objects.get(**kwargs).hub})

        try:
            if hardware_type == 'DHT22':
                hardware_form = HardwareDHT22Form(instance=DHT22.objects.get(**kwargs), initial={'type': 'DHT22'})
            if hardware_type == 'MCP3008':
                hardware_form = HardwareMCP3008Form(instance=MCP3008.objects.get(**kwargs), initial={'type': 'MCP3008'})
            if hardware_type == 'ModProbe':
                hardware_form = HardwareModProbeForm(instance=ModProbe.objects.get(**kwargs),
                                                     initial={'type': 'ModProbe'})
            if hardware_type == 'PiPico':
                hardware_form = HardwarePiPicoForm(instance=PiPico.objects.get(**kwargs), initial={'type': 'PiPico'})
            if hardware_type == 'VEML7700':
                hardware_form = HardwareVEML7700Form(instance=VEML7700.objects.get(**kwargs),
                                                     initial={'type': 'VEML7700'})
            if hardware_type == 'PMSA0031':
                hardware_form = HardwarePMSA0031Form(instance=PMSA0031.objects.get(**kwargs),
                                                     initial={'type': 'PMSA0031'})
        except:
            hardware_form = HardwareForm(instance=Hardware.objects.get(**kwargs))

        channels = Hardware.objects.get(**kwargs).channel_set.all()
        hardware_configs = Hardware.objects.get(**kwargs).hardwareconfig_set.all()
        channel_form = ChannelForm(hardware_type=Hardware.objects.get(**kwargs).type,
                                   initial={'hardware': Hardware.objects.get(**kwargs)})
        hardware_config_form = HardwareConfigForm()
        if request.accepted_renderer.format == 'html':
            return Response(
                {"hardware_form": hardware_form, "channel_form": channel_form, "channels": channels,
                 "hardware_config_form": hardware_config_form,
                 "hardware_configs": hardware_configs, 'hardware_io_type_form': hardware_io_type_form},
                template_name='hardware.html')

    def postChannel(request):
        # request should be ajax and method should be POST.
        if request.is_ajax and request.method == "POST":
            # get the form data
            form = ChannelForm(data=request.POST)
            # save the data and after fetch the object in instance
            if form.is_valid():
                instance = form.save()
                print(str(instance))
                # serialize in new friend object in json
                ser_instance = django_serializers.serialize('json', [instance, ])
                print(str(ser_instance))
                # send to client side.
                return JsonResponse({"instance": ser_instance}, status=200)
            else:
                # some form errors occured.
                return JsonResponse({"error": form.errors}, status=400)

        # some error occured
        return JsonResponse({"error": ""}, status=400)

    def postConfig(request):
        # request should be ajax and method should be POST.
        if request.is_ajax and request.method == "POST":
            # get the form data
            form = HardwareConfigForm(request.POST)
            # save the data and after fetch the object in instance
            if form.is_valid():
                instance = form.save()
                print(str(instance))
                # serialize in new friend object in json
                ser_instance = django_serializers.serialize('json', [instance, ])
                print(str(ser_instance))
                # send to client side.
                return JsonResponse({"instance": ser_instance}, status=200)
            else:
                # some form errors occured.
                return JsonResponse({"error": form.errors}, status=400)

        # some error occured
        return JsonResponse({"error": ""}, status=400)

    def deleteHardware(request):
        # request should be ajax and method should be POST.
        if request.is_ajax and request.method == "POST":
            # get the form data
            hardware = Hardware.objects.get(
                pk=request.POST['id'])  # save the data and after fetch the object in instance
            try:
                from polymorphic.utils import reset_polymorphic_ctype
                hardware.delete()
            except PolymorphicTypeInvalid:

                hardware.get_real_concrete_instance_class().delete()
            return HttpResponse(render(request, 'hardwares.html'))


from data.forms.forms import DataTransform, DataTransformRoot
from data.models.models import DataTransformer
from data.serializers.serializers import DataTransformerTreeSerializer


class AccessoryViewSet(viewsets.ModelViewSet):
    queryset = Accessory.objects.all()
    serializer_class = AccessorySerializer
    renderer_classes = (renderers.JSONRenderer, renderers.TemplateHTMLRenderer)

    def list(self, request, *args, **kwargs):
        print('list')
        response = super(AccessoryViewSet, self).list(request, *args, **kwargs)
        form = AccessoryForm()
        accessories = Accessory.objects.all()
        if request.accepted_renderer.format == 'html':
            return Response({"form": form, "accessories": accessories}, template_name='accessories.html')
        return response

    def retrieve(self, request, *args, **kwargs):
        print('retrieve')
        print(str(request.path_info))

        accessory = Accessory.objects.get(**kwargs)
        calibrations = accessory.calibration_set.all()
        accessory_form = AccessoryForm(instance=accessory)
        calibration_form = CalibrationForm(initial={'accessory': accessory})

        # datatransformerset = inlineformset_factory(DataTransformer, DataTransformer,formset=DataTransformerForm,
        #                                            fk_name='parent', form=DataTransform,extra=0)

        if accessory.has_data_transformer:
            data_transformer = accessory.datatransformer
            formset = DataTransformRoot(instance=data_transformer)
        else:
            data_transformer_root = DataTransformer.objects.create(accessory=accessory)
            data_transformer_root.save()
            formset = DataTransformRoot(instance=data_transformer_root)

        if request.accepted_renderer.format == 'html':
            return Response(
                {"accessory_form": accessory_form, "calibration_form": calibration_form, "calibrations": calibrations,
                 "data_transformer_form": formset},
                template_name='accessory.html')
        else:
            return super(AccessoryViewSet, self).retrieve(request, *args, **kwargs)

    def postAccessory(request):
        # request should be ajax and method should be POST.
        if request.is_ajax and request.method == "POST":
            # get the form data
            form = AccessoryForm(request.POST)
            # save the data and after fetch the object in instance
            if form.is_valid():
                instance = form.save()
                print(str(instance))
                # serialize in new friend object in json
                ser_instance = django_serializers.serialize('json', [instance, ])
                print(str(ser_instance))
                # send to client side.
                return JsonResponse({"instance": ser_instance}, status=200)
            else:
                # some form errors occured.
                return JsonResponse({"error": form.errors}, status=400)

        # some error occured
        return JsonResponse({"error": ""}, status=400)

    def postDataTransformer(request, accessory_id):
        print(str(request.POST))
        print('post data transformer')
        # request should be ajax and method should be POST.
        form = DataTransformRoot(instance=DataTransformer.objects.get(pk=request.POST['id']),
                             data=request.POST)
        if form.is_valid():
            form.save(True)
        else:
            print(form.errors)

        data_transformer_root = form.save()
        data_transformer_family = data_transformer_root.get_family()
        children_dict = defaultdict(list)
        for descendant in data_transformer_family:
            for child in descendant.get_children():
                children_dict[descendant.pk].append(child)

        context = {}
        context['children'] = children_dict
        serializer = DataTransformerTreeSerializer(data_transformer_root, context=context)
        print(str(serializer.data))

        return JsonResponse({"instance": serializer.data}, status=200)


    def postCalibration(request):
        print('post calibration')

        # request should be ajax and method should be POST.
        if request.is_ajax and request.method == "POST":
            # get the form data
            form = CalibrationForm(request.POST)
            # save the data and after fetch the object in instance
            if form.is_valid():
                instance = form.save()
                print(str(instance))
                # serialize in new friend object in json
                ser_instance = django_serializers.serialize('json', [instance, ])
                print(str(ser_instance))
                # send to client side.
                return JsonResponse({"instance": ser_instance}, status=200)
            else:
                # some form errors occured.
                return JsonResponse({"error": form.errors}, status=400)

        # some error occured
        return JsonResponse({"error": ""}, status=400)

    def updateAccessory(request):
        # request should be ajax and method should be POST.
        if request.is_ajax and request.method == "PATCH":
            # get the form data
            form = AccessoryForm(instance=Accessory.objects.get(pk=QueryDict(request.body)['id']),
                                 data=QueryDict(request.body))
            # save the data and after fetch the object in instance
            if form.is_valid():
                instance = form.save()
                print(str(instance))
                # serialize in new friend object in json
                ser_instance = django_serializers.serialize('json', [instance, ])
                print(str(ser_instance))
                # send to client side.
                return JsonResponse({"instance": ser_instance}, status=200)
            else:
                # some form errors occured.
                return JsonResponse({"error": form.errors}, status=400)

        # some error occured
        return JsonResponse({"error": ""}, status=400)

    def deleteAccessory(request):
        # request should be ajax and method should be POST.
        if request.is_ajax and request.method == "DELETE":
            # get the form data
            form = AccessoryForm(instance=Accessory.objects.get(pk=QueryDict(request.body)['id']),
                                 data=QueryDict(request.body))
            accessory = Accessory.objects.get(
                pk=QueryDict(request.body)['id'])  # save the data and after fetch the object in instance
            accessory.delete()

            return HttpResponse(render(request, 'accessories.html'))


class CalibrationViewSet(viewsets.ModelViewSet):
    queryset = Calibration.objects.all()
    serializer_class = CalibrationSerializer
    renderer_classes = (renderers.JSONRenderer, renderers.TemplateHTMLRenderer)

    def retrieve(self, request, *args, **kwargs):
        print('retrieve')
        print(str(request.path_info))
        calibration = Calibration.objects.get(**kwargs)
        calibration_constants = calibration.calibrationconstants_set.all()
        calibration_form = CalibrationForm(instance=calibration)
        calibrationconstants_form = CalibrationConstantForm(initial={'calibration': calibration})

        calibration_constants_forms = []

        for calibration_constant in calibration_constants:
            calibration_constants_forms.append(CalibrationConstantForm(instance=calibration_constant))
        if request.accepted_renderer.format == 'html':
            return Response(
                {"calibration_form": calibration_form, "calibrationconstants_initial_form": calibrationconstants_form,
                 "calibrationconstants_forms": calibration_constants_forms,
                 "calibrations": calibration_constants, "calibration": calibration},
                template_name='calibration.html')
        else:
            return super(CalibrationViewSet, self).retrieve()

    def updateCalibration(request):
        # request should be ajax and method should be POST.
        if request.is_ajax and request.method == "UPDATE":
            # get the form data
            form = CalibrationForm(request.UPDATE)
            # save the data and after fetch the object in instance
            if form.is_valid():
                instance = form.save()
                print(str(instance))
                # serialize in new friend object in json
                ser_instance = django_serializers.serialize('json', [instance, ])
                print(str(ser_instance))
                # send to client side.
                return JsonResponse({"instance": ser_instance}, status=200)
            else:
                # some form errors occured.
                return JsonResponse({"error": form.errors}, status=400)

        # some error occured
        return JsonResponse({"error": ""}, status=400)

    def postCalibrationConstant(request):
        # request should be ajax and method should be POST.
        if request.is_ajax and request.method == "POST":
            # get the form data
            try:
                initial_calibration_constant = CalibrationConstants.objects.get(id=request.POST['id'])
                form = CalibrationConstantForm(instance=initial_calibration_constant, data=request.POST)
            except CalibrationConstants.DoesNotExist:
                form = CalibrationConstantForm(data=request.POST)

            # save the data and after fetch the object in instance
            if form.is_valid():
                instance = form.save()
                print(str(instance))
                # serialize in new friend object in json
                ser_instance = django_serializers.serialize('json', [instance, ])
                form_json = django_serializers.serialize('json', [instance, ])
                print(str(ser_instance))
                # send to client side.
                return JsonResponse({"instance": ser_instance, "form": form_json}, status=200)
            else:
                # some form errors occured.
                return JsonResponse({"error": form.errors}, status=400)

        # some error occured
        return JsonResponse({"error": ""}, status=400)


class ChannelViewSet(viewsets.ModelViewSet):
    queryset = Channel.objects.all()
    serializer_class = ChannelSerializer
    renderer_classes = (renderers.JSONRenderer, renderers.TemplateHTMLRenderer)

    def retrieve(self, request, *args, **kwargs):
        print('retrieve')
        print(str(request.path_info))
        hardware_io_type_form = HardwareIOTypeForm(
            initial={'parent_hardware': Channel.objects.get(**kwargs).hardware.id,
                     'child_channel': Channel.objects.get(**kwargs).id,
                     'hub': Channel.objects.get(**kwargs).hub.id})
        hardware_ios = Channel.objects.get(**kwargs).hardwareio_set.all()

        channel = Channel.objects.get(**kwargs)

        channel_form = ChannelForm(Channel.objects.get(**kwargs).hardware.type, instance=channel)
        if request.accepted_renderer.format == 'html':
            return Response(
                {"channel_form": channel_form, "channel": channel, 'hardware_io_type_form': hardware_io_type_form,
                 'hardware_ios': hardware_ios},
                template_name='channel.html')
        else:
            return super(ChannelViewSet, self).retrieve()

    def listIO(self, channel_id):
        print('list io')
        hardware_ios = Channel.objects.get(pk=channel_id).hardwareio_set.all()
        serialized_hardware_ios = HardwareIOSerializer(many=True).to_representation(hardware_ios)

        return JsonResponse(serialized_hardware_ios, status=200, safe=False)

    def updateChannel(request):
        # request should be ajax and method should be POST.
        if request.is_ajax and request.method == "PATCH":
            # get the form data
            form = ChannelForm(instance=Channel.objects.get(pk=QueryDict(request.body)['id']),
                               data=QueryDict(request.body))

            # save the data and after fetch the object in instance
            if form.is_valid():
                instance = form.save()
                print(str(instance))
                # serialize in new friend object in json
                ser_instance = django_serializers.serialize('json', [instance, ])
                print(str(ser_instance))
                # send to client side.
                return JsonResponse({"instance": ser_instance}, status=200)
            else:
                # some form errors occured.
                return JsonResponse({"error": form.errors}, status=400)

        # some error occured
        return JsonResponse({"error": ""}, status=400)

    def deleteChannel(request):
        # request should be ajax and method should be POST.
        if request.is_ajax and request.method == "DELETE":
            # get the form data
            form = ChannelForm(instance=Channel.objects.get(pk=QueryDict(request.body)['id']),
                               data=QueryDict(request.body))
            channel = Channel.objects.get(
                pk=QueryDict(request.body)['id'])  # save the data and after fetch the object in instance
            channel.delete()

            return HttpResponse(render(request, 'accessories.html'))


class HubViewSet(viewsets.ModelViewSet):
    queryset = Hub.objects.all()
    serializer_class = HubSerializer
    renderer_classes = (renderers.JSONRenderer, renderers.TemplateHTMLRenderer)
    context_object_name = 'all_hubs'

    def get_queryset(self):
        return Hub.objects.all()

    def about(self, *args, **kwargs):
        api_about = {}
        api_about['isAPI'] = True
        return JsonResponse(api_about, status=200, safe=False)

    def list(self, request, *args, **kwargs):
        print('list')
        response = super(HubViewSet, self).list(request, *args, **kwargs)
        form = HubForm()
        hubs = Hub.objects.all()
        if request.accepted_renderer.format == 'html':
            return Response({"form": form, "hubs": hubs}, template_name='hubs.html')
        return response

    @csrf_exempt
    def retrieve(self, request, *args, **kwargs):

        if request.accepted_renderer.format == 'html':
            print('retrieve')
            print(str(request.path_info))
            hub = Hub.objects.get(**kwargs)
            hub_form = HubForm(instance=hub)

            # accessories_form = AccessoryForm()
            # channel_form = ChannelForm(Hub.objects.get(**kwargs).channel_set.type, instance=channel)
            return Response(
                {"channel_form": hub_form},
                template_name='hub.html')
        else:
            hub = Hub.objects.get(**kwargs)
            hub_data = HubSerializer(instance=hub).to_representation(hub)
            return JsonResponse(hub_data, status=200)

    def postHub(request):
        # request should be ajax and method should be POST.
        if request.is_ajax() and request.method == "POST":
            # get the form data
            form = HubForm(request.POST)
            # save the data and after fetch the object in instance
            if form.is_valid():
                instance = form.save()
                print(str(instance))
                # serialize in new friend object in json
                ser_instance = django_serializers.serialize('json', [instance, ])
                print(str(ser_instance))
                # send to client side.
                return JsonResponse({"instance": ser_instance}, status=200)
            else:
                # some form errors occured.
                return JsonResponse({"error": form.errors}, status=400)
        elif request.method == "POST":
            import json
            body_json = json.loads(request.body.decode('utf-8'))
            hub = Hub()
            hub.id = body_json['id']
            hub.ip = body_json['ip']
            hub.display_name = body_json['display_name']
            hub.aid = body_json['aid']
            hub.save()
            ser_instance = django_serializers.serialize('json', [hub, ])

            return JsonResponse({"instance": ser_instance}, status=200)

        # some error occured
        return JsonResponse({"error": ""}, status=400)


def index(request):
    context = {'title': 'HomeHub'}
    return render(request, 'home.html', context)


#


class IOViewSet(viewsets.ModelViewSet):
    queryset = HardwareIO.objects.all()
    serializer_class = HardwareIOSerializer
    renderer_classes = (renderers.JSONRenderer, renderers.TemplateHTMLRenderer)

    def retrieve(self, request, *args, **kwargs):
        print('retrieve')
        print(str(request.path_info))
        hardware_io_type = HardwareIO.objects.get(**kwargs).type
        instance = HardwareIO.objects.get(**kwargs)
        if hardware_io_type == SPI:
            hardware_form = SPIIoForm(instance=instance)
        if hardware_io_type == Serial:
            hardware_form = SerialIoForm(instance=instance)
        if hardware_io_type == PWM:
            hardware_form = PwmIoForm(instance=instance)
        if hardware_io_type == I2C:
            hardware_form = I2cIoForm(instance=instance)
        if hardware_io_type == DeviceFile:
            hardware_form = DeviceFileIoForm(instance=instance)
        if hardware_io_type == MCPChannel:
            hardware_form = MCPAnalogIoForm(instance=instance)
        if hardware_io_type == PiPicoAnalog:
            hardware_form = PiPicoAnalogIoForm(instance=instance)
        if hardware_io_type == PiPicoACAnalog:
            hardware_form = PiPicoACAnalogIoForm(instance=instance)
        if hardware_io_type == PiGPIO:
            hardware_form = PiGpioForm(instance=instance)

        if request.accepted_renderer.format == 'html':
            return Response(
                {"hardware_io_form": hardware_form},
                template_name='io.html')
        else:
            return super(IOViewSet, self).retrieve()

    def updateIO(request):
        # request should be ajax and method should be POST.
        if request.is_ajax and request.method == "PATCH":
            # get the form data
            form = HardwareIoForm(instance=Channel.objects.get(pk=QueryDict(request.body)['id']),
                                  data=QueryDict(request.body))

            # save the data and after fetch the object in instance
            if form.is_valid():
                instance = form.save()
                print(str(instance))
                # serialize in new friend object in json
                ser_instance = django_serializers.serialize('json', [instance, ])
                print(str(ser_instance))
                # send to client side.
                return JsonResponse({"instance": ser_instance}, status=200)
            else:
                # some form errors occured.
                return JsonResponse({"error": form.errors}, status=400)

        # some error occured
        return JsonResponse({"error": ""}, status=400)

    def deleteIO(request):
        # request should be ajax and method should be POST.
        if request.is_ajax and request.method == "DELETE":
            # get the form data
            form = HardwareIoForm(instance=HardwareIO.objects.get(pk=QueryDict(request.body)['id']),
                                  data=QueryDict(request.body))
            hardware_io = HardwareIO.objects.get(
                pk=QueryDict(request.body)['id'])  # save the data and after fetch the object in instance
            hardware_io.delete()

            return HttpResponse(render(request, 'accessories.html'))


def video_streams(request):
    return render(request, "streams.html", None)


from django.contrib.auth import login, authenticate
from django.contrib.auth.forms import UserCreationForm
from django.shortcuts import render, redirect


def signup(request):
    if request.method == 'POST':
        form = UserCreationForm(request.POST)
        if form.is_valid():
            form.save()
            username = form.cleaned_data.get('username')
            raw_password = form.cleaned_data.get('password1')
            user = authenticate(username=username, password=raw_password)
            login(request, user)
            return redirect('hubs')
    else:
        form = UserCreationForm()
    return render(request, 'registration/signup.html', {'form': form})


@api_view(('GET',))
@renderer_classes((TemplateHTMLRenderer, JSONRenderer))
def listHubHardware(request, hub_id):
    form_type = HardwareTypeForm(initial={'type': 'PiPico'})
    hardware_form = HardwarePiPicoForm(initial={'type': 'PiPico'})
    hardware = Hub.objects.get(pk=hub_id).hardware_set.all()
    if request.accepts("text/html"):
        return Response({"form": hardware_form, "type_form": form_type, "hardwares": hardware},
                        template_name='hardwares.html')

    serialized_hardware = HardwareSerializer().to_json_array(hardware)
    return JsonResponse(serialized_hardware, status=200, safe=False)


@api_view(('GET',))
@renderer_classes((TemplateHTMLRenderer, JSONRenderer))
def listHubChannels(request, hub_id):
    channels = Hub.objects.get(pk=hub_id).channel_set.all()
    serialized_channels = ChannelSerializer(many=True).to_representation(channels)
    return JsonResponse(serialized_channels, status=200, safe=False)


@api_view(('GET',))
@renderer_classes((TemplateHTMLRenderer, JSONRenderer))
def listHubAccessories(request, hub_id):
    accessories = Hub.objects.get(pk=hub_id).accessory_set.all()

    if request.accepts("text/html"):
        form = AccessoryForm()
        return Response({"form": form, "accessories": accessories}, template_name='accessories.html')
    serialized_accessories = AccessorySerializer(many=True).to_representation(accessories)

    return JsonResponse(serialized_accessories, status=200, safe=False)



# Generated by Django 3.2.5 on 2021-08-13 23:36

from django.db import migrations
def gen(apps, schema_editor):
    from data.models.models import AccessoryType, HardwareChannelTypes, Category

    types = ['Relay']

    for type in types:
        cat = AccessoryType.objects.create()
        cat.type = type
        cat.save()

    hardware_channel_types = [('PiPico', 'PiPicoRelay')]

    for (hard, channel) in hardware_channel_types:
        cat = HardwareChannelTypes.objects.create()
        cat.hardware_type = hard
        cat.channel_type = channel
        cat.save()


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0014_auto_20210721_0457'),
    ]

    operations = [
        migrations.RunPython(gen, reverse_code=migrations.RunPython.noop),
    ]



"""
ASGI config for OpenHubAPI project.

It exposes the ASGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/3.2/howto/deployment/asgi/
"""

import os

from channels.auth import AuthMiddlewareStack
from channels.routing import ProtocolTypeRouter, URLRouter
from django.core.asgi import get_asgi_application
from . import routing

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'settings')


application = ProtocolTypeRouter({
  "http": get_asgi_application(),
  "websocket": AuthMiddlewareStack(
        URLRouter(
            routing.websocket_urlpatterns
        )
    ),
})

# application = get_asgi_application()
# mysite/asgi.py



# chat/consumers.py
import json
from channels.generic.websocket import WebsocketConsumer


class CalibrateConsumer(WebsocketConsumer):
    def connect(self):
        self.accept()

    def disconnect(self, close_code):
        pass

    def receive(self, text_data):
        text_data_json = json.loads(text_data)
        message = text_data_json['message']

        self.send(text_data=json.dumps({
            'message': message
        }))



# Generated by Django 3.2.4 on 2021-06-10 18:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('data', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='dht22',
            options={'base_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='mcp3008',
            options={'base_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='modprobe',
            options={'base_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='pipico',
            options={'base_manager_name': 'objects'},
        ),
        migrations.AlterModelOptions(
            name='veml7700',
            options={'base_manager_name': 'objects'},
        ),
        migrations.AddField(
            model_name='hardware',
            name='polymorphic_ctype',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='polymorphic_data.hardware_set+', to='contenttypes.contenttype'),
        ),
    ]



# Generated by Django 3.2.4 on 2021-06-11 21:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0006_accessorytype'),
    ]

    operations = [
        migrations.AddField(
            model_name='pipico',
            name='serial_com',
            field=models.CharField(max_length=100, null=True),
        ),
    ]



# Generated by Django 3.2.5 on 2021-10-04 19:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0029_alter_datatransformer_parent'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='datatransformer',
            name='parent',
        ),
    ]



# Generated by Django 3.2.5 on 2021-10-03 03:11

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0026_auto_20211003_0308'),
    ]

    operations = [
        migrations.AddField(
            model_name='datatransformer',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='datatransformer',
            name='updated_at',
            field=models.DateTimeField(auto_now=True),
        ),
    ]



# Generated by Django 3.2.4 on 2021-07-13 15:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0012_hardwareio_type'),
    ]

    operations = [
        migrations.RenameField(
            model_name='pwmio',
            old_name='pwm_pin',
            new_name='en',
        ),
        migrations.AddField(
            model_name='hardwareio',
            name='label',
            field=models.CharField(max_length=255, null=True),
        ),
    ]



# Generated by Django 3.2.4 on 2021-07-12 20:38

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0009_auto_20210712_1926'),
    ]

    operations = [
        migrations.CreateModel(
            name='PiPins',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('pin', models.IntegerField()),
                ('hardware_io', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='data.hardwareio')),
            ],
        ),
        migrations.AddIndex(
            model_name='pipins',
            index=models.Index(fields=['hardware_io'], name='data_pipins_hardwar_586dc9_idx'),
        ),
    ]



# Generated by Django 3.2.4 on 2021-07-12 19:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0008_auto_20210712_1649'),
    ]

    operations = [
        migrations.AddField(
            model_name='pigpio',
            name='channel_index',
            field=models.IntegerField(null=True),
        ),
        migrations.AddField(
            model_name='pipicoanalogio',
            name='pin',
            field=models.IntegerField(default=0),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.2.5 on 2021-08-14 18:02

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0016_auto_20210814_1801'),
    ]

    operations = [
        migrations.CreateModel(
            name='PMSA0031',
            fields=[
                ('hardware_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='data.hardware')),
                ('scl', models.IntegerField()),
                ('sda', models.IntegerField()),
                ('reset', models.IntegerField()),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
            bases=('data.hardware',),
        ),
    ]



# Generated by Django 3.2.4 on 2021-06-11 18:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0004_category'),
    ]

    operations = [
        migrations.AlterField(
            model_name='category',
            name='enum',
            field=models.IntegerField(null=True),
        ),
    ]



# Generated by Django 3.2.5 on 2021-08-14 18:01

from django.db import migrations
def gen(apps, schema_editor):
    from data.models.models import AccessoryType, HardwareChannelTypes, Category

    types = ['Camera']

    for type in types:
        cat = AccessoryType.objects.create()
        cat.type = type
        cat.save()



class Migration(migrations.Migration):

    dependencies = [
        ('data', '0015_auto_20210813_2336'),
    ]

    operations = [
        migrations.RunPython(gen, reverse_code=migrations.RunPython.noop),
    ]



"""OpenHubAPI URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.2/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.conf.urls import url
from django.contrib import admin
from django.urls import include
from django.views.decorators.csrf import csrf_exempt
from rest_framework import routers
from django.urls import path
from data.views.views import HardwareViewSet, AccessoryViewSet, CalibrationViewSet, ChannelViewSet, HubViewSet, \
    IOViewSet, signup,listHubAccessories,listHubHardware,listHubChannels
from django.contrib.auth.views import LoginView


# Routers provide an easy way of automatically determining the URL conf.
router = routers.DefaultRouter()
router.register(r'accessories', AccessoryViewSet)
router.register(r'hardwares', HardwareViewSet)
router.register(r'channel', ChannelViewSet)
router.register(r'calibration', CalibrationViewSet)
router.register(r'hubs', HubViewSet, basename='hubs')
router.register(r'io', IOViewSet)

from data.views.views import index
from data.views.views import video_streams

# Wire up our API using automatic URL routing.
# Additionally, we include login URLs for the browsable API.
urlpatterns = [
    path('', include(router.urls)),
    url(r'^login/$', LoginView.as_view(template_name="registration/login.html"), name="login"),
    path('api-auth/', include('rest_framework.urls', namespace='rest_framework')),
    # path('start', views.start, name='start'),
    # path('calibrate', views.start, name='calibrate'),
    path('index', index, name='index'),
    url(r'^admin/', admin.site.urls),
    path('post/ajax/hardware', HardwareViewSet.postHardware, name="post_hardware"),
    path('post/ajax/hardware/form_type/', HardwareViewSet.getHardwareTypeForm,
         name="get_hardware_types"),
    path('post/ajax/hardware/form_io/', HardwareViewSet.getHardwareIoForm,
         name="get_hardware_io"),
    path('post/ajax/accessory', AccessoryViewSet.postAccessory, name="post_accessory"),
    path('delete/ajax/accessory', AccessoryViewSet.deleteAccessory, name="delete_accessory"),
    path('post/ajax/calibration', AccessoryViewSet.postCalibration, name="post_calibration"),
    path('patch/ajax/accessory', AccessoryViewSet.updateAccessory, name="update_accessory"),
    path('patch/ajax/calibration', CalibrationViewSet.updateCalibration, name="update_calibration"),
    path('delete/ajax/channel', ChannelViewSet.deleteChannel, name="delete_channel"),
    path('delete/ajax/hardware', HardwareViewSet.deleteHardware, name="delete_hardware"),
    path('delete/ajax/io', IOViewSet.deleteIO, name="delete_io"),

    path('post/ajax/hardware/io/', HardwareViewSet.postHardwareIO,
         name="post_hardware_io"),
    path('post/ajax/calibrationconstant', CalibrationViewSet.postCalibrationConstant, name="post_calibration_constant"),
    path('post/ajax/channel', HardwareViewSet.postChannel, name="post_channel"),
    path('patch/ajax/channel', ChannelViewSet.updateChannel, name="update_channel"),
    path('post/ajax/config', HardwareViewSet.postConfig, name="post_config"),
    path('patch/ajax/hardware', HardwareViewSet.updateHardware, name="update_hardware"),
    path('hardwares/<uuid:hardware_id>/config', HardwareViewSet.getConfig),
    path('hardwares/<uuid:hardware_id>/channels', HardwareViewSet.getChannels),
    path('channels/<uuid:channel_id>/io', ChannelViewSet.listIO),
    path('accessories/<uuid:accessory_id>/data_transformer', AccessoryViewSet.postDataTransformer),

    # path('<str:room_name>/', views.room, name='room'),
    path('post/ajax/hub', HubViewSet.postHub, name="post_hub"),
    path('hub/', csrf_exempt(HubViewSet.postHub)),
    path('pipico/', csrf_exempt(HardwareViewSet.postHardware)),

    path('hubs/<uuid:hub_id>/hardwares', view=listHubHardware),
    path('hubs/<uuid:hub_id>/channels', view=listHubChannels),
    path('hubs/<uuid:hub_id>/accessories', view=listHubAccessories),
    path('openhubapi/about', HubViewSet.about),
    path('streams/', video_streams, name="video_streams"),
    url(r'^signup/$', signup, name='signup'),

]



# Generated by Django 3.2.5 on 2021-10-06 17:45

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0038_auto_20211006_1745'),
    ]

    operations = [
        migrations.AlterField(
            model_name='datatransformerconstants',
            name='id',
            field=models.UUIDField(blank=True, default=uuid.UUID('254e53c8-9256-4f60-bf6e-42862dbd6a11'), primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='hardwarestatsdatatransformer',
            name='id',
            field=models.UUIDField(blank=True, default=uuid.UUID('9c02df0d-3861-4989-95af-89f4537f7393'), primary_key=True, serialize=False),
        ),
    ]



from rest_framework import serializers

from data.models.models import CalibrationConstants, Calibration, Channel, Accessory, HardwareConfig, PiPico, \
    DHT22, MCP3008, ModProbe, VEML7700, Hardware, Hub, SPIIo, SerialIo, PwmIo, I2cIo, DeviceFileIo, MCPAnalogIo, \
    PiPicoAnalogIo, PiPicoACAnalogIo, PiGpio, DataTransformer,DataTransformerConstants

class RecursiveField(serializers.Serializer):
    def to_representation(self, value):
        serializer = self.parent.parent.__class__(value, context=self.context)
        return serializer.data

class CalibrationConstantsSerializer(serializers.ModelSerializer):
    class Meta:
        model = CalibrationConstants
        fields = ['id', 'type', 'value']


class HubSerializer(serializers.ModelSerializer):
    class Meta:
        model = Hub
        fields = '__all__'


class CalibrationSerializer(serializers.ModelSerializer):
    calibration_constants = CalibrationConstantsSerializer(source='calibrationconstants_set', many=True,
                                                           read_only=False)

    class Meta:
        model = Calibration
        fields = ['id', 'type', 'calibration_constants']


class ChannelSerializer(serializers.ModelSerializer):
    class Meta:
        model = Channel
        fields = ['id', 'type', 'channel_index', 'hardware']


class AccessorySerializer(serializers.ModelSerializer):
    # channels = ChannelSerializer(source='channel_set', many=True)
    calibration = CalibrationSerializer(source='calibration_set', many=True, read_only=False)

    class Meta:
        model = Accessory
        fields = ['id', 'category', 'type', 'display_name', 'aid',
                  'calibration', 'channels']


class DataTransformerSerializer(serializers.ModelSerializer):
    data_transformer_child = RecursiveField(many=True)

    class Meta:
        model = DataTransformer
        fields = '__all__'


class HardwareConfigSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = HardwareConfig
        fields = ['id', 'type', 'value']


class PiPicoSerializer(serializers.ModelSerializer):
    class Meta:
        model = PiPico
        fields = '__all__'


class DHT22Serializer(serializers.ModelSerializer):
    class Meta:
        model = DHT22
        fields = '__all__'


class MCP3008Serializer(serializers.ModelSerializer):
    class Meta:
        model = MCP3008
        fields = '__all__'


class ModProbeSerializer(serializers.ModelSerializer):
    class Meta:
        model = ModProbe
        fields = '__all__'


class VEML7700Serializer(serializers.ModelSerializer):
    class Meta:
        model = VEML7700
        fields = '__all__'


class HardwareSerializer(serializers.ModelSerializer):

    def to_json_array(self, obt_list):
        serialized_hardware = []
        for obj in obt_list:
            serialized_hardware.append(self.to_representation(obj))
        return serialized_hardware

    def to_representation(self, obj):
        """
        Because GalleryItem is Polymorphic
        """
        if isinstance(obj, PiPico):
            return PiPicoSerializer(obj, context=self.context).to_representation(obj)
        elif isinstance(obj, DHT22):
            return DHT22Serializer(obj, context=self.context).to_representation(obj)
        elif isinstance(obj, MCP3008):
            return MCP3008Serializer(obj, context=self.context).to_representation(obj)
        elif isinstance(obj, ModProbe):
            return ModProbeSerializer(obj, context=self.context).to_representation(obj)
        elif isinstance(obj, VEML7700):
            return VEML7700Serializer(obj, context=self.context).to_representation(obj)
        return super(HardwareSerializer, self).to_representation(obj)

    class Meta:
        model = Hardware
        fields = '__all__'


class SPIIoSerializer(serializers.ModelSerializer):
    class Meta:
        model = SPIIo
        fields = '__all__'


class SerialIoSerializer(serializers.ModelSerializer):
    class Meta:
        model = SerialIo
        fields = '__all__'


class PwmIoSerializer(serializers.ModelSerializer):
    class Meta:
        model = PwmIo
        fields = '__all__'


class I2cIoSerializer(serializers.ModelSerializer):
    class Meta:
        model = I2cIo
        fields = '__all__'


class DeviceFileIoSerializer(serializers.ModelSerializer):
    class Meta:
        model = DeviceFileIo
        fields = '__all__'


class MCPAnalogIoSerializer(serializers.ModelSerializer):
    class Meta:
        model = MCPAnalogIo
        fields = '__all__'


class PiPicoAnalogIoSerializer(serializers.ModelSerializer):
    class Meta:
        model = PiPicoAnalogIo
        fields = '__all__'


class PiPicoACAnalogIoSerializer(serializers.ModelSerializer):
    class Meta:
        model = PiPicoACAnalogIo
        fields = '__all__'


class PiGpioSerializer(serializers.ModelSerializer):
    class Meta:
        model = PiGpio
        fields = '__all__'


class HardwareIOSerializer(serializers.ModelSerializer):

    def to_json_array(self, obt_list):
        serialized_hardware = []
        for obj in obt_list:
            serialized_hardware.append(self.to_representation(obj))
        return serialized_hardware

    def to_representation(self, obj):
        """
        Because GalleryItem is Polymorphic
        """
        if isinstance(obj, SPIIo):
            return SPIIoSerializer(obj, context=self.context).to_representation(obj)
        elif isinstance(obj, SerialIo):
            return SerialIoSerializer(obj, context=self.context).to_representation(obj)
        elif isinstance(obj, PwmIo):
            return PwmIoSerializer(obj, context=self.context).to_representation(obj)
        elif isinstance(obj, I2cIo):
            return I2cIoSerializer(obj, context=self.context).to_representation(obj)
        elif isinstance(obj, DeviceFileIo):
            return DeviceFileIoSerializer(obj, context=self.context).to_representation(obj)
        elif isinstance(obj, MCPAnalogIo):
            return MCPAnalogIoSerializer(obj, context=self.context).to_representation(obj)

        elif isinstance(obj, PiPicoAnalogIo):
            return PiPicoAnalogIoSerializer(obj, context=self.context).to_representation(obj)
        elif isinstance(obj, PiPicoACAnalogIo):
            return PiPicoACAnalogIoSerializer(obj, context=self.context).to_representation(obj)
        elif isinstance(obj, PiGpio):
            return PiGpioSerializer(obj, context=self.context).to_representation(obj)

        return super(HardwareIOSerializer, self).to_representation(obj)

    class Meta:
        model = Hardware
        fields = '__all__'


class DataTransformerConstantsSerializer(serializers.ModelSerializer):
    datatransformer = serializers.PrimaryKeyRelatedField(many=True, read_only=True)

    class Meta:
        model = DataTransformerConstants
        fields = '__all__'



class DataTransformerTreeSerializer(serializers.ModelSerializer):
    data_transformer_constants = DataTransformerConstantsSerializer(many=True)
    children = serializers.SerializerMethodField(source='get_children')
    class Meta:
        model=DataTransformer
        fields = '__all__'  # add here rest of the fields from model

    def get_children(self, obj):
        children = self.context['children'].get(obj.id, [])
        serializer = DataTransformerTreeSerializer(children, many=True, context=self.context)
        return serializer.data


# Generated by Django 3.2.5 on 2021-10-04 17:53

from django.db import migrations, models
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0027_auto_20211003_0311'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='datatransformer',
            name='data_transformer_parent',
        ),
        migrations.AddField(
            model_name='datatransformer',
            name='level',
            field=models.PositiveIntegerField(default=1, editable=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='datatransformer',
            name='lft',
            field=models.PositiveIntegerField(default=1, editable=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='datatransformer',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='data.datatransformer'),
        ),
        migrations.AddField(
            model_name='datatransformer',
            name='rght',
            field=models.PositiveIntegerField(default=1, editable=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='datatransformer',
            name='tree_id',
            field=models.PositiveIntegerField(db_index=True, default=1, editable=False),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='datatransformer',
            name='accessory',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='data.accessory'),
        ),
    ]



# Generated by Django 3.2.4 on 2021-06-11 18:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0003_auto_20210610_2329'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, null=True)),
                ('enum', models.IntegerField()),
            ],
        ),
    ]
